import static java.lang.Math.*;


public class atomFunc{
    static vectorFunc v;
    
    public atomFunc()
    {
        v = new vectorFunc();
    }
    
    public static atom zeroforce(atom a)    // reset force accumulator
    {
        atom b = new atom(a);
        b.force = new vector(0,0);	
        return b;
    }
    
	public static float distance(atom a, atom b)	// find distance between atoms
    {
        return v.mag(v.sub(a.pos, b.pos));
    }
	
	public static vector separate(atom a)		// return randomized separation force
    {
        return v.scale(new vector(Math.random()-0.5f, Math.random()-0.5f), Math.random()*100*a.mass);
    }
	
	public static atom addforce(atom a, vector f)	// accumulate force
	{
		atom b = new atom(a);
		b.force = v.add(b.force, f);
		return b;
	}
	
	public static atom subforce(atom a, vector f)	// accumulate force
	{
		atom b = new atom(a);
		b.force = v.sub(b.force, f);
		return b;
	}
	
	public static float sizes(atom a, atom b)	// return combined radius of two atoms
	{
		return a.radius+b.radius;
	}
                                           
	public static vector repulse(atom a, atom b, float dist)	// return self repulsion spring+damper forces
	{
		vector spring = v.scale((v.sub(b.pos, a.pos)), (dist-(a.radius+b.radius))*90000);   // K=5000
		vector damper = v.scale((v.sub(b.vel, a.vel)), 1);										// D=20
		return v.add(spring, damper);
	}
	
    public static atom gravity(atom a, vector grav)		// apply gravity on atom
    {
        atom b = new atom(a);
		b.force = v.add(b.force, v.scale(grav, b.mass));
		return b;
    }
    
    public static atom integrate(atom a, float dt)		// integrate system
    {
        atom b = new atom(a);
		b.vel = v.add(b.vel, v.scale(v.invscale(b.force, b.mass), dt));    // v = (f/m) *dt   or   v = a*dt  or  v = integrated v
		b.pos = v.add(b.pos, v.scale(b.vel, dt));                    // x += vel *dt
		
		return b;
    }
	
	public static atom hardcollision(atom a, vector boxSize, vector boxPos, float decay, float friction) // only for 0' and 90' angles
    {
        atom b = new atom(a);
		
		if (b.pos.x-b.radius < boxPos.x)     //else if can miss corner collisions
		{
			b.pos = new vector(boxPos.x+b.radius, b.pos.y);
			b.vel = new vector(b.vel.x * -decay, b.vel.y * friction);	
		}
		if (b.pos.x+b.radius > boxPos.x+boxSize.x)
		{
			b.pos = new vector(boxPos.x+boxSize.x-b.radius, b.pos.y);
			b.vel = new vector(b.vel.x * -decay, b.vel.y * friction);
		}
		if (b.pos.y-b.radius < boxPos.y)
		{
			b.pos = new vector(b.pos.x, boxPos.y+b.radius);
			b.vel = new vector(b.vel.x * friction, b.vel.y * -decay);
		}
		if (b.pos.y+b.radius > boxPos.y+boxSize.y)
		{
			b.pos = new vector(b.pos.x, boxPos.y+boxSize.y-b.radius);
			b.vel = new vector(b.vel.x * friction, b.vel.y * -decay);
		}
		return b;
    }
	
	public static atom terminalvelocity(atom a)		// apply gravity on atom
    {
        atom b = new atom(a);
		if (b.vel.x > 5000)
		{
			b.vel = new vector(5000,b.vel.y);
		}
		if (b.vel.x < -5000)
		{
			b.vel = new vector(-5000,b.vel.y);
		}
		if (b.vel.y > 5000)
		{
			b.vel = new vector(b.vel.x,5000);
		}
		if (b.vel.y < -5000)
		{
			b.vel = new vector(b.vel.x,-5000);
		}
		return b;
    }
	
	public static atom velocitydrag(atom a)		// apply gravity on atom
    {
        atom b = new atom(a);
		b.vel = v.scale(b.vel, 0.98f);
		return b;
    }
	
	public static float getmass(atom a)    // reset force accumulator
    {       
        return a.mass;
    }
};
